@@ -12,8 +12,6 @@ C_LIBRARY <- "MonetDB.R"
 DEBUG_IO      <- FALSE
 DEBUG_QUERY   <- FALSE
 
-
-
 # Make S4 aware of S3 classes
 setOldClass(c("sockconn","connection","monetdb_mapi_conn"))
 
@@ -27,21 +25,24 @@ MonetR <- MonetDB <- MonetDBR <- MonetDB.R <- function() {
 
 setMethod("dbGetInfo", "MonetDBDriver", def=function(dbObj, ...)
 			list(name="MonetDBDriver", 
-					driver.version="0.8.0",
+					driver.version="0.9",
 					DBI.version="0.2-5",
 					client.version=NA,
 					max.connections=NA)
 )
 
 # shorthand for connecting to the DB, very handy, e.g. dbListTables(mc("acs"))
-mc <- function(dbname="demo", user="monetdb", password="monetdb", host="localhost",port=50000, timeout=86400, wait=FALSE,...) {
-	dbConnect(MonetDB.R(),dbname,user,password,host,port,timeout,wait,...)
+mc <- function(dbname="demo", user="monetdb", password="monetdb", host="localhost",port=50000L, timeout=86400L, wait=FALSE,language="sql",...) {
+	dbConnect(MonetDB.R(),dbname,user,password,host,port,timeout,wait,language,...)
 }
 
-setMethod("dbConnect", "MonetDBDriver", def=function(drv,dbname="demo", user="monetdb", password="monetdb", host="localhost",port=50000, timeout=86400, wait=FALSE,...,url="") {
+setMethod("dbConnect", "MonetDBDriver", def=function(drv,dbname="demo", user="monetdb", password="monetdb", host="localhost",port=50000L, timeout=86400L, wait=FALSE,language="sql",...,url="") {
 			if (substring(url,1,10) == "monetdb://") {
 				dbname <- url
 			}
+			port <- as.integer(port)
+			timeout <- as.integer(timeout)
+			
 			if (substring(dbname,1,10) == "monetdb://") {
 				#warning("MonetDB.R: Using 'monetdb://...' URIs in dbConnect() is deprecated. Please switch to dbname, host, port named arguments.")
 				rest <- substring(dbname,11,nchar(dbname))
@@ -71,9 +72,10 @@ setMethod("dbConnect", "MonetDBDriver", def=function(drv,dbname="demo", user="mo
 								#	blocking = TRUE, open="r+b",timeout = 5 )
 								
 								# this goes to src/mapi.c
+								# TODO: this ends up in the global environment. Bad.
 								socket <- socket <<- .Call("mapiConnect",host,port,5,PACKAGE=C_LIBRARY)
 								# authenticate
-								.monetAuthenticate(socket,dbname,user,password)
+								.monetAuthenticate(socket,dbname,user,password,language=language)
 								# test the connection to make sure it works before
 								.mapiWrite(socket,"sSELECT 42;"); .mapiRead(socket)
 								#close(socket)
@@ -94,7 +96,7 @@ setMethod("dbConnect", "MonetDBDriver", def=function(drv,dbname="demo", user="mo
 			#socket <- socket <<- socketConnection(host = host, port = port, 
 			#	blocking = TRUE, open="r+b",timeout = timeout) 
 			socket <- socket <<- .Call("mapiConnect",host,port,timeout,PACKAGE=C_LIBRARY)
-			.monetAuthenticate(socket,dbname,user,password)
+			.monetAuthenticate(socket,dbname,user,password,language=language)
 			connenv <- new.env(parent=emptyenv())
 			connenv$lock <- 0
 			connenv$deferred <- list()
@@ -110,7 +112,7 @@ setClass("MonetDBConnection", representation("DBIConnection",socket="externalptr
 
 setMethod("dbDisconnect", "MonetDBConnection", def=function(conn, ...) {
 			.Call("mapiDisconnect",conn@socket,PACKAGE=C_LIBRARY)
-			TRUE
+			return(invisible(TRUE))
 		})
 
 setMethod("dbListTables", "MonetDBConnection", def=function(conn, ...) {
@@ -126,7 +128,7 @@ setMethod("dbListFields", "MonetDBConnection", def=function(conn, name, ...) {
 		})
 
 setMethod("dbExistsTable", "MonetDBConnection", def=function(conn, name, ...) {
-			name %in% dbListTables(conn)
+			tolower(name) %in% dbListTables(conn)
 		})
 
 
@@ -153,7 +155,11 @@ setMethod("dbSendQuery", signature(conn="MonetDBConnection", statement="characte
 			if(!is.null(list) || length(list(...))){
 				if (length(list(...))) statement <- .bindParameters(statement, list(...))
 				if (!is.null(list)) statement <- .bindParameters(statement, list)
-			}		
+			}	
+
+# removeme
+#write(statement,file="/export/scratch2/hannes/anthony-joinbug/log.sql",append=TRUE)
+
 			conn@connenv$exception <- list()
 			env <- NULL
 			if (DEBUG_QUERY)  cat(paste("QQ: '",statement,"'\n",sep=""))
@@ -235,7 +241,7 @@ setMethod("dbWriteTable", "MonetDBConnection", def=function(conn, name, value, o
 				dbSendQuery(conn,"COMMIT")
 				.mapiRequest(conn, "Xauto_commit 1")
 			}
-			TRUE
+			return(invisible(TRUE))
 		})
 
 
@@ -245,15 +251,16 @@ setMethod("dbDataType", signature(dbObj="MonetDBConnection", obj = "ANY"), def =
 			else if (is.numeric(obj)) "DOUBLE PRECISION"
 			else if (is.raw(obj)) "BLOB"
 			
-			else "VARCHAR(255)"
+			else "STRING"
 		}, valueClass = "character")
 
 
 setMethod("dbRemoveTable", "MonetDBConnection", def=function(conn, name, ...) {
-			if (dbExistsTable(conn,name)) 
-				dbSendUpdate(conn, paste("DROP TABLE", name))
-			return(TRUE)
-			return(FALSE)
+			if (dbExistsTable(conn,name)) {
+				dbSendUpdate(conn, paste("DROP TABLE", tolower(name)))
+				return(invisible(TRUE))
+			}
+			return(invisible(FALSE))
 		})
 
 # for compatibility with RMonetDB (and dbWriteTable support), we will allow parameters to this method, but will not use prepared statements internally
@@ -267,7 +274,7 @@ setMethod("dbSendUpdate", signature(conn="MonetDBConnection", statement="charact
 			if (!res@env$success) {
 				stop(paste(statement,"failed!\nServer says:",res@env$message))
 			}
-			TRUE
+			return(invisible(TRUE))
 		})
 
 # this can be used in finalizers to not mess up the socket
@@ -412,7 +419,7 @@ setMethod("fetch", signature(res="MonetDBResult", n="numeric"), def=function(res
 			#parts[parts=="NULL"] <- NA
 			
 			# call to a faster C implementation for the hard and annoying task of splitting everyting into fields
-			parts <- .Call("mapiSplit", res@env$data[1:n],info$cols, PACKAGE=C_LIBRARY)
+			parts <- .Call("mapiSplit", res@env$data[1:n],as.integer(info$cols), PACKAGE=C_LIBRARY)
 			
 			# convert values column by column
 			for (j in seq.int(info$cols)) {	
@@ -642,8 +649,8 @@ REPLY_SIZE    <- 100 # Apparently, -1 means unlimited, but we will start with a
 			env$types	<- env$dbtypes <- toupper(.mapiParseTableHeader(lines[4]))
 			env$lengths	<- .mapiParseTableHeader(lines[5])
 			
-			env$tuples <-lines[6:length(lines)]
-
+			if (env$rows > 0) env$tuples <- lines[6:length(lines)]
+			
 			stopifnot(length(env$tuples) == header$index)
 			return(env)
 		}
@@ -711,7 +718,7 @@ REPLY_SIZE    <- 100 # Apparently, -1 means unlimited, but we will start with a
 # on each connection. The server starts by sending a challenge, to which we respond by
 # hashing our login information using the server-requested hashing algorithm and its salt.
 
-.monetAuthenticate <- function(con,dbname,user="monetdb",password="monetdb",endhashfunc="sha512") {
+.monetAuthenticate <- function(con,dbname,user="monetdb",password="monetdb",endhashfunc="sha512",language="sql") {
 	endhashfunc <- tolower(endhashfunc)
 	# read challenge from server, it looks like this
 	# oRzY7XZr1EfNWETqU6b2:merovingian:9:RIPEMD160,SHA256,SHA1,MD5:LIT:SHA512:
@@ -730,7 +737,7 @@ REPLY_SIZE    <- 100 # Apparently, -1 means unlimited, but we will start with a
 	salt <- credentials[[1]][1]
 	
 	if (!(endhashfunc %in% HASH_ALGOS)) {
-		stop(paste("Hash function",pwhashfunc,"is not available on server"))
+	  stop(paste("Server-requested end hash function",endhashfunc,"is not available"))
 	}
 	
 	if (!(pwhashfunc %in% HASH_ALGOS)) {
@@ -746,8 +753,8 @@ REPLY_SIZE    <- 100 # Apparently, -1 means unlimited, but we will start with a
 	
 	# we respond to the authentication challeng with something like
 	# LIT:monetdb:{SHA512}eec43c24242[...]cc33147:sql:acs
-	# endianness:username:passwordhash:scenario:databasename
-	authString <- paste("LIT:",user,":{",toupper(endhashfunc),"}",hashsum,":sql:",dbname,":",sep="")
+	# endianness:username:passwordhash:language:databasename
+	authString <- paste0("LIT:",user,":{",toupper(endhashfunc),"}",hashsum,":",language,":",dbname,":")
 	.mapiWrite(con,authString)
 	authResponse <- .mapiRead(con)
 	respKey <- substring(authResponse,1,1)
@@ -789,11 +796,9 @@ REPLY_SIZE    <- 100 # Apparently, -1 means unlimited, but we will start with a
 }
 
 # copied from RMonetDB, no java-specific things in here...
-# TODO: read first few rows with read.table and check types etc.
-
-monet.read.csv <- monetdb.read.csv <- function(conn,files,tablename,nrows,header=TRUE,locked=FALSE,na.strings="",...,nrow.check=500,delim=","){
+monet.read.csv <- monetdb.read.csv <- function(conn,files,tablename,nrows,header=TRUE,locked=FALSE,na.strings="",nrow.check=500,delim=",",newline="\\n",quote="\"",...){
 	if (length(na.strings)>1) stop("na.strings must be of length 1")
-	headers<-lapply(files,read.csv,na.strings="NA",...,nrows=nrow.check)
+	headers<-lapply(files,read.csv,sep=delim,na.strings=na.strings,quote=quote,nrows=nrow.check,...)
 	
 	if (length(files)>1){
 		nn<-sapply(headers,ncol)
@@ -805,17 +810,19 @@ monet.read.csv <- monetdb.read.csv <- function(conn,files,tablename,nrows,header
 	} 
 	
 	dbWriteTable(conn, tablename, headers[[1]][FALSE,])
+
+	delimspec <- paste0("USING DELIMITERS '",delim,"','",newline,"','",quote,"'")
 	
 	if(header || !missing(nrows)){
 		if (length(nrows)==1) nrows<-rep(nrows,length(files))
 		for(i in seq_along(files)) {
 			cat(files[i],thefile<-normalizePath(files[i]),"\n")
-			dbSendUpdate(conn, paste("copy",format(nrows[i],scientific=FALSE),"offset 2 records into", tablename,"from",paste("'",thefile,"'",sep=""),"using delimiters ',','\\n','\"' NULL as",paste("'",na.strings[1],"'",sep=""),if(locked) "LOCKED"))
+			dbSendUpdate(conn, paste("COPY",format(nrows[i],scientific=FALSE),"OFFSET 2 RECORDS INTO", tablename,"FROM",paste("'",thefile,"'",sep=""),delimspec,"NULL as",paste("'",na.strings[1],"'",sep=""),if(locked) "LOCKED"))
 		}
 	} else {
 		for(i in seq_along(files)) {
 			cat(files[i],thefile<-normalizePath(files[i]),"\n")
-			dbSendUpdate(conn, paste0("copy into ", tablename," from ",paste("'",thefile,"'",sep="")," using delimiters '",delim,"','\\n','\"' NULL as ",paste("'",na.strings[1],"'",sep=""),if(locked) " LOCKED "))
+			dbSendUpdate(conn, paste0("COPY INTO ", tablename," FROM ",paste("'",thefile,"'",sep=""),delimspec,"NULL as ",paste("'",na.strings[1],"'",sep=""),if(locked) " LOCKED "))
 		}
 	}
 	dbGetQuery(conn,paste("select count(*) from",tablename))
@@ -826,6 +833,53 @@ counterenv$bytes.in <-  	counterenv$bytes.out <- numeric(1)
 
 monetdbGetTransferredBytes <- function() {
 	ret <- list(bytes.in=counterenv$bytes.in,bytes.out=counterenv$bytes.out)
-	counterenv$bytes.in <-  	counterenv$bytes.out <- numeric(1)
+	counterenv$bytes.in <- counterenv$bytes.out <- numeric(1)
 	ret
 }
+
+.monetdbd.command <- function(passphrase,host="localhost",port=50000L,timeout=86400L) {
+  socket <- .Call("mapiConnect",host,port,timeout,PACKAGE=C_LIBRARY)
+  .monetAuthenticate(socket,"merovingian","monetdb",passphrase,language="control")
+  .mapiWrite(socket,"#all status\n")
+  ret <- .mapiRead(socket)
+  .Call("mapiDisconnect",socket,PACKAGE=C_LIBRARY)
+  return (ret)
+}
+
+monetdbd.liststatus <- monetdb.liststatus <- function(passphrase,host="localhost",port=50000L,timeout=86400L) {
+  rawstr <- .monetdbd.command(passphrase,host,port,timeout)
+  lines <- strsplit(rawstr,"\n",fixed=T)[[1]] # split by newline, first line is "=OK", so skip
+  lines <- lines[grepl("^=sabdb:2:",lines)] # make sure we get a db list here, protocol v.2
+  lines <- sub("=sabdb:2:","",lines,fixed=T)
+  # convert value into propert types etc
+  dbdf <- as.data.frame(do.call("rbind",strsplit(lines,",",fixed=T)),stringsAsFactors=F)
+  names(dbdf) <- c("dbname","uri","locked","state","scenarios","startCounter","stopCounter","crashCounter","avgUptime","maxUptime","minUptime","lastCrash","lastStart","lastStop","crashAvg1","crashAvg10","crashAvg30")
+  
+  dbdf$locked <- dbdf$locked=="1"
+  
+  states <- c("illegal","running","crashed","inactive","starting")
+  dbdf$state <- factor(states[as.integer(dbdf$state)+1])
+  
+  dbdf$startCounter <- as.numeric(dbdf$startCounter)
+  dbdf$stopCounter <- as.numeric(dbdf$stopCounter)
+  dbdf$crashCounter <- as.numeric(dbdf$crashCounter)
+  
+  dbdf$avgUptime <- as.numeric(dbdf$avgUptime)
+  dbdf$maxUptime <- as.numeric(dbdf$maxUptime)
+  dbdf$minUptime <- as.numeric(dbdf$minUptime)
+  
+  convertts <- function(col) {
+    col[col=="-1"] <- NA
+    return(as.POSIXct(as.numeric(col), origin="1970-01-01"))
+  }
+  dbdf$lastCrash <- convertts(dbdf$lastCrash)
+  dbdf$lastStart <- convertts(dbdf$lastStart)
+  dbdf$lastStop <- convertts(dbdf$lastStop)
+  
+  dbdf$crashAvg1 <- dbdf$crashAvg1=="1"
+  dbdfcrashAvg10 <- as.numeric(dbdf$crashAvg10)
+  dbdf$crashAvg30 <- as.numeric(dbdf$crashAvg30)
+  dbdf$scenarios <- gsub("'",",",dbdf$scenarios,fixed=T)
+  
+  return(dbdf[order(dbdf$dbname),])
+}
